from pandas import DataFrame
import pandas as pd
from plyer import filechooser
import re

class df_processor(DataFrame):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

# method import csv werkt ! :)
    def processor_import_csv(self, *separators, encoding = 'UTF-8'):
        """
        Import a CSV file using a pop up window, you can give 1 or more separators and a specific encoding.
        If the separator in your csv file is different than ',' use one argument like: ';'
        If there are two or more different separators use two or more arguments like: ',' , ';'

        No arguments given: default separator ',' and default encoding 'UTF-8' will be used

        :param separators: The separator(s) used in the CSV file you want to import
        :type separators: string
    
        :param encoding: The encoding used in the CSV file
        :type encoding: string

        :return: The function will return a Pandas DataFrame with the data in the CSV
        :type return: Pandas DataFrame
        """

        import_path = filechooser.open_file()
    
        if import_path:
            if separators:
                sep_pattern = '|'.join(map(lambda s: re.escape(str(s)), separators))

            else:
                sep_pattern = ','

            self = pd.read_csv(import_path[0], sep=sep_pattern, engine='python', encoding = encoding)
            print('csv file is imported')
            self = df_processor(self)
            return self 
        else:
            print("No file selected.")


#file exporter for csv-file werkt ! :)
    def processor_export_csv(self, sep= ','):
    
        """
    export your pandas dataframe as csv file with seperator of your choice with a pop-up window
    in pop-up window write the name of your file + .csv
    
    no arguments given: default seperator ','
    
    :param df: the pandas dataframe you want to export to csv-file
    :type df: pandas dataframe
    
    :param sep: the seperator you would like your exported csv-file to have
    :type sep: string
    
    :return: pandas dataframe and an exported csv-file
    :type return: pandas dataframe
   
    """

        export_path = filechooser.save_file()
        if export_path:
            self = super().to_csv(export_path[0], sep= sep , index = False)
            return self
        else:
            print("No file saved")


# method delete nan values  werkt :)      
    def delete_is_na(self):
   
        """
   
    deletes all rows in pandas dataframe where value in every column is nan.
   
    :param df: pandas dataframe in which you would like to delete all rows with nan values
    :type df: pandas dataframe
   
    :return: pandas dataframe without rows with nan values
    :type return: pandas dataframe
   
    """
   
    
        nan_rows = self[super().isna().all(axis=1)]
      
        df_cleaned = self.drop(nan_rows.index)
        num_NaN_removed = len(self) - len(df_cleaned)
        print(f"Removed {num_NaN_removed} rows.")
        df_cleaned = df_processor(df_cleaned)
        return df_cleaned
    


# method duplicaten checken
    def duplicate_rows_check(self):
        """
    Check if there are duplicate rows in a DataFrame.
 
    :param df: The Pandas DataFrame you want to check
    :type df: Pandas DataFrame
 
    :return: The function will return an overview of the duplicate rows with a linebreak after every set of duplicates.
    :type return: string
    """
 
        df_sorted = super().sort_values(by=super().columns.tolist()) 
        duplicate_rows = df_sorted[df_sorted.duplicated(keep=False)]
    
        if not duplicate_rows.empty:
            print("Duplicate rows found:")
            prev_row = None
            for index, row in duplicate_rows.iterrows():
                if prev_row is None or not row.equals(prev_row):
                    print("  ")
                print(index, "\t", " ".join(row.astype(str)))
                prev_row = row
        else:
            print("No duplicate rows found")


  
# method duplicate rijen verwijderen
    def delete_duplicate_rows(self):
   
        """
   
    deletes all duplicate rows that are found in the pandas dataframe
   
    :param df: pandas dataframe in which you would like to delete duplicate rows
    :type df: pandas dataframe
   
    :return: pandas dataframe where all duplicate rows are deleted
    :type return: pandas dataframe
   
    """
   
        df_cleaned = self.drop_duplicates(keep='first')
   
        num_duplicates_removed = len(self) - len(df_cleaned)
        print(f"Removed {num_duplicates_removed} duplicate rows.")
        df_cleaned = df_processor(df_cleaned)
        return df_cleaned
    

# method delete duplicate kolommen 
    def delete_duplicate_column(self, reference_column, column_to_delete):
        """
    When all the values are equal in both columns, the colomn_to_delete will be deleted
 
    :param df: The Pandas DataFrame
    :type df: Pandas DataFrame
 
    :param reference_column: The column you want to check and keep
    :type reference_column: string
 
    :param df: The column you want to check and delete if both columns are the same
    :type column_to_: string
    
    :return: The function will return the DataFrame without the column_to_delete or a message when the columns are not identical
    :type return: string
    """
    
    
        if (self[reference_column] == self[column_to_delete]).all():
            self = super().drop(column_to_delete, axis=1)
            self = df_processor(self)
            return self
        else:
            print(f"The {reference_column} and {column_to_delete} columns do not have the same values.")
        


    def nan_check(self):
        """
    Check if there are NaN values in a DataFrame.

 

    :param df: The Pandas DataFrame you want to check.
    :type df: Pandas DataFrame

 

    :return: This function does not return a value; it prints an overview of the NaN values
             with their corresponding row number and column.
    :rtype: None
    """
        missing_rows = self[super().isnull().any(axis=1)]
        if not missing_rows.empty:
            print("Rows with missing values:")
            for index, row in missing_rows.iterrows():
                missing_columns = row.index[row.isnull()]
                print(f"Row {index}:")
                for col in missing_columns:
                    print(f"   {col}: {row[col]}")
        else:
            print("No rows with missing values found.")

        



   
